﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;

/* TellCore Solutions LLC -- UE Multiplexer example control code.
 * v1.0  10/14
 * 
 * This project is intended as a simple illustration of how one might code for control of the UE Multiplexer.
 * 
 * It provides an example of 
 *  - opening a socket connection with the unit;
 *  - sending the unit a sample of the basic operational commands;
 *  - parsing the return string to confirm successful acceptance of the commands;
 *  - closing the socket connection; 
 *  
 * Additionally it shares an example of how to change the IP Mode and address via the Telnet interface.
 * The project assumes the unit is configured for DHCP Client and currently has an allocated address.
 * The IP Settings change example 
 *  - allows the IP mode to be changed to STATIC with a chosen fixed Address;
 *  - the user may then use the first example to connect and issue commands 
 *      to the new address (after a reboot);
 *  - the IP Settings can then be reverted back to DHCP Client;
 *  - the user may then use the first example to connect and issue commands 
 *      to the re-allocated address (after a reboot);
 * 
 * A default set of IP settings assuming a 10.16.0.xxx network are provided here. THe user should
 * change these as required for his particular circumstances.
 * 
 * There is a very simple GUI to trigger the various example actions and provide appropriate indications to the user. 
 */

namespace TelnetExample
{
    class TelnetTest
    {
        // assumes initial IP address based upon a DHCP allocation
        const string IP_ADDR_FOR_DHCP_TEST = "10.16.0.108"; // identify your current assigned IP address and edit here appropriately.
        const int DEFAULT_TELNET_PORT_NUM = 9760;

        const string IP_ADDR_FOR_STATIC_TEST = "10.16.0.121";
        const string SUBNET_FOR_STATIC_TEST = "255.255.255.0";
        const string GATEWAY_FOR_STATIC_TEST = "10.16.0.1";

        bool m_UseDHCPAddress = true;
        string m_IpAddr = IP_ADDR_FOR_DHCP_TEST;
        int m_TelnetPort = DEFAULT_TELNET_PORT_NUM;
        
        Socket m_MySocket;
        IPAddress m_ServIPAddr;
        IPEndPoint m_ServEndPoint;
        
        // open a socket and connect to the Telnet port
        public bool Connect()
        {
            if(m_UseDHCPAddress)
                m_IpAddr = IP_ADDR_FOR_DHCP_TEST;
            else
                m_IpAddr = IP_ADDR_FOR_STATIC_TEST;

            m_MySocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            m_ServIPAddr = IPAddress.Parse(m_IpAddr);
            m_ServEndPoint = new IPEndPoint(m_ServIPAddr, m_TelnetPort);

            try
            {
                m_MySocket.Connect(m_ServEndPoint);

                byte[] rBuf = new byte[255];
                int rcvCnt = m_MySocket.Receive(rBuf); // clear out any content - initial Prompt
                //MessageBox.Show("Successful connection to the UE Mux!");
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                return (false);
            }
            return (true);
        }

        // issue a series of commands to the unit
        public void SendExampleCommands()
        {
            // Select UE #3 as the Active UE
            string testCmd = "SELECT=3\r\n"; // All capitalized; no spaces; CR+LF;
            SendCommand(testCmd);

            // Provide USB power only to the ACTIVE UE
            testCmd = "POWER=ACTIVE\r\n"; // All capitalized; no spaces; CR+LF;
            SendCommand(testCmd);
        
            // Connect the Active UE 
            testCmd = "CONNECT=1\r\n"; // All capitalized; no spaces; CR+LF;
            SendCommand(testCmd);
        }

        // Send the command via Telnet and accept any return string
        public bool SendCommand(string Command)
        {
            bool retStatus = true; // prime for success
                        
            byte[] tBuf = Encoding.Default.GetBytes(Command);
            m_MySocket.Send(tBuf);

            // Successful return string would be expected to include: "OK\r\nUEMux>"
            byte[] rBuf = new byte[255];
            int rcvCnt = m_MySocket.Receive(rBuf);


            string rcvStr = System.Text.Encoding.UTF8.GetString(rBuf, 0, rcvCnt);// convert to string for easy manipulation
            if (!rcvStr.Contains("OK")) // Confirm success
            {
                MessageBox.Show("Command error: " + rcvStr);
                retStatus = false;
            }

            return (retStatus);
        }

        // Disconnect the socket.
        public void Disconnect()
        {
            if((m_MySocket != null) && (m_MySocket.Connected))
                m_MySocket.Disconnect(false);
            
        }



        

        //////////////////////////////////////////////////////////////
        // This example shows how one might code for configuring a new IP Mode and IP address setting.
        // All such changes require a reboot of the unit to take effect.
        // 
        // Send the command to request Static IP address assignment upon next power cycling of the unit, and provide the address.
        public void ChangeIPToStaticAddressing()
        {
            // select an available IP address on the same subnet (i.e. one that does not respond to a ping)
            string testIPAddr = IP_ADDR_FOR_STATIC_TEST;
            string testSNetMask = SUBNET_FOR_STATIC_TEST;
            string testGateway = GATEWAY_FOR_STATIC_TEST;

            // Configure IP Mode for static, update the IP address to an available static IP Address and reset the unit.
            string testCmd = "IP_MODE=STATIC\r\n"; // All capitalized; no spaces;
            SendCommand(testCmd);

            testCmd = "IP_ADDR=" + testIPAddr;  // initial parameter with no space
            testCmd += ", " + testSNetMask;     // additional parameter separated by comma and space
            testCmd += ", " + testGateway;      // additional parameter separated by comma and space
            testCmd += "\r\n";                  // end with CR+LF

            if (SendCommand(testCmd))
            {
                // if successful, save the new IP address, disconnect and encourage a reset of the unit 
                // to have the new IP configuration take hold.
                m_IpAddr = testIPAddr;
                Disconnect();
                MessageBox.Show("Reset the UE Mux to enforce the new IP configuration.");
            }

            m_UseDHCPAddress = false; // ensure next connection uses this new static IP            

        }

        // Send the command to request DHCP Client mode for IP address assignment upon next power cycling of the unit.
        public void ChangeIPToDHCPClient()
        {
            // Configure IP Mode for DHCP Client and reset the unit.
            string testCmd = "IP_MODE=CLIENT\r\n"; // All capitalized; no spaces;
            if (SendCommand(testCmd))
            {
                // if successful, disconnect and encourage a reset of the unit 
                // to have the new IP configuration take hold.
                m_IpAddr = IP_ADDR_FOR_DHCP_TEST; // prime for same address, on the likelihood of a re-assignemnt to the same as initially
                Disconnect();
                MessageBox.Show("Reset the UE Mux to enforce the new IP configuration.");
            }

            m_UseDHCPAddress = true; // revert to DHCP Client mode 
        }


    }
}
